@@ -341,16 +341,20 @@ template <class Qtype> class Quantum : public QBase{
   // Get value in canonical base units
   Qtype getBaseValue() const;
 
-  // Get value in specified units. If the <src>other</src> units do not conform to
-  // the units of this object and requireConform is True, an exception is thrown,
-  // with the following excepions:
-  // angle to/from time conversions are implicitly supported, frequency to/from
-  // wavelength conversions are implicitly supported.
-  // Note, I added requireConform and made the default value False for backward
-  // compatibility. However, I think that ultimately requireConform should be removed
-  // and an exception should be thrown if the units do not conform. It's not clear to
-  // me what this was not in the original implementation; it's much to easy for
-  // non-conformation bugs to slip by unnoticed. - dmehring 09feb2015
+  // Get value in specified units.
+  // If the <src>other</src> units do not conform to the units of this
+  // object and requireConform is True, an exception is thrown,
+  // with the following exceptions:
+  // <br>- angle to/from time conversions are implicitly supported
+  // <br>- frequency to/from/ wavelength conversions are implicitly supported
+  //# <br>Note, I added requireConform and made the default value False for
+  //# backward compatibility. However, I think that ultimately requireConform
+  //# should be removed and an exception should be thrown if the units do
+  //# not conform. It's not clear to me why this was not in the original
+  //# implementation; it's much too easy for non-conformation bugs to
+  //# slip by unnoticed. - dmehring 09feb2015
+  //# It should be left in since conversion from time to angle makes sense.
+  //# Maybe the default could be changed to True. - gvandiepen09feb2016
   Qtype getValue(const Unit &other, Bool requireConform=False) const;
 
   // Get the unit (as Unit) that is attached to the Quantum. (use getUnit() if
@@ -437,6 +441,11 @@ Bool readQuantity(Quantity &res, MUString &in);
 Bool readQuantity(Quantity &res, const String &in);
 // </group>
 
+//# Declare extern templates for often used types.
+#ifdef AIPS_CXX11
+  extern template class Quantum<Double>;
+#endif
+
 
 } //# NAMESPACE CASACORE - END
 
