@@ -461,7 +461,11 @@ protected:
 // </group>
 
 
-
+//# Declare extern templates for often used types.
+#ifdef AIPS_CXX11
+  extern template class PagedImage<Float>;
+  extern template class PagedImage<Complex>;
+#endif
 
 
 } //# NAMESPACE CASACORE - END
