@@ -35,6 +35,7 @@
 #include <casacore/ms/MeasurementSets/MeasurementSet.h>
 #include <casacore/ms/MeasurementSets/MSPointingColumns.h>
 #include <casacore/casa/Utilities/CountedPtr.h>
+#include <casacore/tables/Tables/TableProxy.h>
 #include <map>
 
 namespace casacore {
@@ -48,7 +49,12 @@ struct SubScanKey;
 // <summary>
 // Class to interrogate  an MS for metadata. Interrogation happens on demand
 // and resulting metadata are stored for use by subsequent queries if the
-// cache has not exceeded the specified limit.
+// cache has not exceeded the specified limit. Caching of MS main table columns
+// has been removed because the cache can be swamped by columns for large
+// MSes, meaning that smaller data structures, which are more computationally
+// expensive to create, aren't cached. Also, the column data is usually only
+// needed temporarily to compute smaller data structures, and the column data
+// is not particularly expensive to recreate if necessary.
 // Parallel processing is enabled using openmp.
 // </summary>
 
@@ -345,13 +351,13 @@ public:
     // get the map of scans to time ranges.
     SHARED_PTR<const std::map<ScanKey, std::pair<Double,Double> > > getScanToTimeRangeMap() const;
 
-    // get the times for the specified scan
-    // std::set<Double> getTimesForScan(const uInt scan) const;
-
     // get the stateIDs associated with the specified scan. If obsID and/or arrayID
     // is negative, all observation IDs and/or array IDs will be used.
     std::set<Int> getStatesForScan(Int obsID, Int arrayID, Int scan) const;
 
+    // get a map of spectral windows to unique timestamps.
+    std::vector<std::set<Double> > getTimesForSpws(Bool showProgress=True) const;
+
     // get the position of the specified antenna relative to the observatory position.
     // the three vector returned represents the longitudinal, latitudinal, and elevation
     // offsets (elements 0, 1, and 2 respectively). The longitude and latitude offsets are
@@ -733,8 +739,6 @@ private:
     mutable std::set<String> _uniqueIntents;
     mutable std::set<Int>  _uniqueFieldIDs, _uniqueStateIDs, _uniqueAntennaIDs;
     mutable std::set<uInt> _avgSpw, _tdmSpw, _fdmSpw, _wvrSpw, _sqldSpw, _uniqueDataDescIDs;
-    mutable SHARED_PTR<Vector<Int> > _antenna1, _antenna2, _scans, _fieldIDs,
-        _stateIDs, _dataDescIDs, _observationIDs, _arrayIDs;
     mutable SHARED_PTR<std::map<SubScanKey, uInt> > _subScanToNACRowsMap, _subScanToNXCRowsMap;
     mutable SHARED_PTR<std::map<Int, uInt> > _fieldToNACRowsMap, _fieldToNXCRowsMap;
     mutable std::map<ScanKey, std::set<String> > _scanToIntentsMap;
@@ -750,8 +754,6 @@ private:
     mutable vector<vector<Int> > _corrTypes;
     mutable vector<Array<Int> >_corrProds;
 
-    mutable SHARED_PTR<Vector<Double> > _times;
-    mutable SHARED_PTR<Quantum<Vector<Double> > > _exposures, _intervals;
     mutable SHARED_PTR<std::map<ScanKey, std::set<Double> > > _scanToTimesMap;
     std::map<String, std::set<uInt> > _intentToSpwsMap;
     mutable std::map<String, std::set<Double> > _intentToTimesMap;
@@ -769,7 +771,6 @@ private:
     mutable SHARED_PTR<std::map<SubScanKey, Double> > _unflaggedSubScanNACRows, _unflaggedSubScanNXCRows;
     const String _taqlTableName;
     const vector<const Table*> _taqlTempTable;
-    mutable SHARED_PTR<ArrayColumn<Bool> > _flagsColumn;
 
     mutable Bool _spwInfoStored, _forceSubScanPropsToCache;
     vector<std::map<Int, Quantity> > _firstExposureTimeMap;
@@ -827,6 +828,29 @@ private:
         Bool showProgress
     ) const;
 
+    static void _getScalarIntColumn(
+        Vector<Int>& v, TableProxy& table, const String& colname,
+        Int beginRow, Int nrows
+    );
+
+    static void _getScalarDoubleColumn(
+        Vector<Double>& v, TableProxy& table, const String& colname,
+        Int beginRow, Int nrows
+    );
+
+    static void _getScalarQuantDoubleColumn(
+        Quantum<Vector<Double> >& v, TableProxy& table, const String& colname,
+        Int beginRow, Int nrows
+    );
+
+    void _mergeScanProps(
+        SHARED_PTR<std::map<ScanKey, MSMetaData::ScanProperties> >& scanProps,
+        SHARED_PTR<std::map<SubScanKey, MSMetaData::SubScanProperties> >& subScanProps,
+        const std::vector<
+            pair<map<ScanKey, ScanProperties>, map<SubScanKey, SubScanProperties> >
+        >&  props
+    ) const;
+
     void _createScanRecords(
         Record& parent, const ArrayKey& arrayKey,
         const std::map<SubScanKey, SubScanProperties>& subScanProps
@@ -877,12 +901,12 @@ private:
     // Uses openmp for parallel processing
     pair<std::map<ScanKey, ScanProperties>, std::map<SubScanKey, SubScanProperties> >
     _getChunkSubScanProperties(
-        SHARED_PTR<const Vector<Int> > scans, SHARED_PTR<const Vector<Int> > fields,
-        SHARED_PTR<const Vector<Int> > ddIDs, SHARED_PTR<const Vector<Int> > states,
-        SHARED_PTR<const Vector<Double> > times, SHARED_PTR<const Vector<Int> > arrays,
-        SHARED_PTR<const Vector<Int> > observations, SHARED_PTR<const Vector<Int> > ant1,
-        SHARED_PTR<const Vector<Int> > ant2, SHARED_PTR<const Quantum<Vector<Double> > > exposureTimes,
-        SHARED_PTR<const Quantum<Vector<Double> > > intervalTimes, const vector<uInt>& ddIDToSpw,
+        const Vector<Int>& scans, const Vector<Int>& fields,
+        const Vector<Int>& ddIDs, const Vector<Int>& states,
+        const Vector<Double>& times, const Vector<Int>& arrays,
+        const Vector<Int>& observations, const Vector<Int>& ant1,
+        const Vector<Int>& ant2, const Quantum<Vector<Double> >& exposureTimes,
+        const Quantum<Vector<Double> >& intervalTimes, const vector<uInt>& ddIDToSpw,
         uInt beginRow, uInt endRow
     ) const;
 
