@@ -64,6 +64,9 @@ public:
         const ChauvenetCriterionStatistics<CASA_STATP>& other
     );
 
+    // Clone this instance
+    virtual StatisticsAlgorithm<CASA_STATP>* clone() const;
+ 
     // get the algorithm that this object uses for computing stats
     virtual StatisticsData::ALGORITHM algorithm() const {
         return StatisticsData::CHAUVENETCRITERION;
