@@ -108,6 +108,9 @@ public:
 
     virtual ~StatisticsAlgorithm();
 
+    // Clone this instance
+    virtual StatisticsAlgorithm<CASA_STATP>* clone() const = 0;
+
     // <group>
     // Add a dataset to an existing set of datasets on which statistics are
     // to be calculated. nr is the number of points to be considered.
@@ -314,6 +317,10 @@ protected:
         return _dataProvider;
     }
 
+    const StatsDataProvider<CASA_STATP>* _getDataProvider() const {
+        return _dataProvider;
+    }
+
     const vector<uInt>& _getDataStrides() const { return _dataStrides; }
 
     const std::map<uInt, Bool>& _getIsIncludeRanges() const { return _isIncludeRanges; }
