@@ -234,12 +234,12 @@ namespace casacore { //# NAMESPACE CASACORE - BEGIN
     virtual DComplex getDComplex (const vector<TableExprId>& = vector<TableExprId>());
     virtual MVTime getDate (const vector<TableExprId>& = vector<TableExprId>());
     virtual String getString (const vector<TableExprId>& = vector<TableExprId>());
-    virtual Array<Bool> getArrayBool (const vector<TableExprId>& = vector<TableExprId>());
-    virtual Array<Int64> getArrayInt (const vector<TableExprId>& = vector<TableExprId>());
-    virtual Array<Double> getArrayDouble (const vector<TableExprId>& = vector<TableExprId>());
-    virtual Array<DComplex> getArrayDComplex (const vector<TableExprId>& = vector<TableExprId>());
-    virtual Array<MVTime> getArrayDate (const vector<TableExprId>& = vector<TableExprId>());
-    virtual Array<String> getArrayString (const vector<TableExprId>& = vector<TableExprId>());
+    virtual MArray<Bool> getArrayBool (const vector<TableExprId>& = vector<TableExprId>());
+    virtual MArray<Int64> getArrayInt (const vector<TableExprId>& = vector<TableExprId>());
+    virtual MArray<Double> getArrayDouble (const vector<TableExprId>& = vector<TableExprId>());
+    virtual MArray<DComplex> getArrayDComplex (const vector<TableExprId>& = vector<TableExprId>());
+    virtual MArray<MVTime> getArrayDate (const vector<TableExprId>& = vector<TableExprId>());
+    virtual MArray<String> getArrayString (const vector<TableExprId>& = vector<TableExprId>());
     // <group>
   private:
     // Copying is not needed, thus not allowed.
@@ -294,12 +294,12 @@ namespace casacore { //# NAMESPACE CASACORE - BEGIN
     virtual DComplex getDComplex (const vector<TableExprId>&);
     virtual MVTime getDate (const vector<TableExprId>&);
     virtual String getString (const vector<TableExprId>&);
-    virtual Array<Bool> getArrayBool (const vector<TableExprId>&);
-    virtual Array<Int64> getArrayInt (const vector<TableExprId>&);
-    virtual Array<Double> getArrayDouble (const vector<TableExprId>&);
-    virtual Array<DComplex> getArrayDComplex (const vector<TableExprId>&);
-    virtual Array<MVTime> getArrayDate (const vector<TableExprId>&);
-    virtual Array<String> getArrayString (const vector<TableExprId>&);
+    virtual MArray<Bool> getArrayBool (const vector<TableExprId>&);
+    virtual MArray<Int64> getArrayInt (const vector<TableExprId>&);
+    virtual MArray<Double> getArrayDouble (const vector<TableExprId>&);
+    virtual MArray<DComplex> getArrayDComplex (const vector<TableExprId>&);
+    virtual MArray<MVTime> getArrayDate (const vector<TableExprId>&);
+    virtual MArray<String> getArrayString (const vector<TableExprId>&);
   protected:
     TableExprId itsId;
   };
@@ -362,7 +362,7 @@ namespace casacore { //# NAMESPACE CASACORE - BEGIN
     virtual ~TableExprGroupRowid();
     virtual Bool isLazy() const;
     virtual void apply (const TableExprId& id);
-    virtual Array<Int64> getArrayInt (const vector<TableExprId>&);
+    virtual MArray<Int64> getArrayInt (const vector<TableExprId>&);
   };
 
   // <summary>
@@ -382,15 +382,15 @@ namespace casacore { //# NAMESPACE CASACORE - BEGIN
     virtual ~TableExprGroupAggr();
     virtual Bool isLazy() const;
     virtual void apply (const TableExprId& id);
-    virtual Array<Bool> getArrayBool (const vector<TableExprId>&);
-    virtual Array<Int64> getArrayInt (const vector<TableExprId>&);
-    virtual Array<Double> getArrayDouble (const vector<TableExprId>&);
-    virtual Array<DComplex> getArrayDComplex (const vector<TableExprId>&);
-    virtual Array<MVTime> getArrayDate (const vector<TableExprId>&);
-    virtual Array<String> getArrayString (const vector<TableExprId>&);
+    virtual MArray<Bool> getArrayBool (const vector<TableExprId>&);
+    virtual MArray<Int64> getArrayInt (const vector<TableExprId>&);
+    virtual MArray<Double> getArrayDouble (const vector<TableExprId>&);
+    virtual MArray<DComplex> getArrayDComplex (const vector<TableExprId>&);
+    virtual MArray<MVTime> getArrayDate (const vector<TableExprId>&);
+    virtual MArray<String> getArrayString (const vector<TableExprId>&);
   protected:
     template<typename T>
-    Array<T> getArray (const vector<TableExprId>& ids)
+    MArray<T> getArray (const vector<TableExprId>& ids)
     {
       // Return scalar values as a Vector.
       if (itsOperand->valueType() == TableExprNodeRep::VTScalar) {
@@ -398,25 +398,57 @@ namespace casacore { //# NAMESPACE CASACORE - BEGIN
         for (size_t i=0; i<ids.size(); ++i) {
           itsOperand->get (ids[i], result[i]);
         }
-        return result;
+        return MArray<T>(result);
       }
       // Array values are returned as an array with one more axis.
-      // Get the first value to determine the shape.
-      Array<T> arr;
-      itsOperand->get (ids[0], arr);
-      IPosition shp = arr.shape();
-      shp.append (IPosition (1, ids.size()));
+      // Use the first non-null value to determine the shape and if masked.
+      MArray<T> arr;
+      size_t id;
+      Bool hasMask = False;
+      IPosition shp;
+      for (id=0; id<ids.size(); ++id) {
+        itsOperand->get (ids[id], arr);
+        if (! arr.isNull()) {
+          hasMask = arr.hasMask();
+          shp = arr.shape();
+          shp.append (IPosition (1, ids.size()));
+          break;
+        }
+      }
+      size_t ndef = 0;
+      if (id == ids.size()) {
+        // All arrays are null.
+        return MArray<T>();
+      }
       Array<T> result(shp);
       ArrayIterator<T> iter (result, arr.ndim());
-      iter.array() = arr;
-      iter.next();
-      int i=1;
-      while (! iter.pastEnd()) {
-        itsOperand->get (ids[i], iter.array());
-        iter.next();
-        i++;
+      Array<Bool> mask;
+      CountedPtr<ArrayIterator<Bool> > miter;
+      if (hasMask) {
+        mask.resize (shp);
+        miter = new ArrayIterator<Bool> (mask, arr.ndim());
       }
-      return result;
+      for (; id<ids.size(); ++id) {
+        MArray<T> values;
+        itsOperand->get (ids[id], values);
+        if (! values.isNull()) {
+          ndef++;
+          iter.array() = values.array();
+          iter.next();
+          if (hasMask) {
+            miter->array() = values.mask();
+            miter->next();
+          }
+        }
+      }
+      if (ndef < ids.size()) {
+        shp[shp.size() - 1] = ndef;
+        result.resize (shp, True);
+        if (hasMask) {
+          mask.resize (shp, True);
+        }
+      }
+      return MArray<T>(result, mask);
     }
   };
 
@@ -605,12 +637,12 @@ namespace casacore { //# NAMESPACE CASACORE - BEGIN
       : TableExprGroupFuncBase (node)
     {}
     virtual ~TableExprGroupFuncArrayBool();
-    virtual Array<Bool> getArrayBool (const vector<TableExprId>&);
+    virtual MArray<Bool> getArrayBool (const vector<TableExprId>&);
   protected:
     // If not empty, check if the shape matches that of <src>itsValue</src>.
     // If <src>itsValue</src> is still empty, it is sized.
-    Bool checkShape (const ArrayBase& arr, const String& func);
-    Array<Bool> itsValue;
+    Bool checkShape (const MArrayBase& arr, const String& func);
+    MArray<Bool> itsValue;
   };
 
   // <summary>
@@ -633,12 +665,12 @@ namespace casacore { //# NAMESPACE CASACORE - BEGIN
       : TableExprGroupFuncBase (node)
     {}
     virtual ~TableExprGroupFuncArrayInt();
-    virtual Array<Int64> getArrayInt (const vector<TableExprId>&);
+    virtual MArray<Int64> getArrayInt (const vector<TableExprId>&);
   protected:
     // If not empty, check if the shape matches that of <src>itsValue</src>.
     // If <src>itsValue</src> is still empty, it is sized.
-    Bool checkShape (const ArrayBase& arr, const String& func);
-    Array<Int64> itsValue;
+    Bool checkShape (const MArrayBase& arr, const String& func);
+    MArray<Int64> itsValue;
   };
 
   // <summary>
@@ -661,12 +693,12 @@ namespace casacore { //# NAMESPACE CASACORE - BEGIN
       : TableExprGroupFuncBase (node)
     {}
     virtual ~TableExprGroupFuncArrayDouble();
-    virtual Array<Double> getArrayDouble (const vector<TableExprId>&);
+    virtual MArray<Double> getArrayDouble (const vector<TableExprId>&);
   protected:
     // If not empty, check if the shape matches that of <src>itsValue</src>.
     // If <src>itsValue</src> is still empty, it is sized.
-    Bool checkShape (const ArrayBase& arr, const String& func);
-    Array<Double> itsValue;
+    Bool checkShape (const MArrayBase& arr, const String& func);
+    MArray<Double> itsValue;
   };
 
   // <summary>
@@ -689,12 +721,12 @@ namespace casacore { //# NAMESPACE CASACORE - BEGIN
       : TableExprGroupFuncBase (node)
     {}
     virtual ~TableExprGroupFuncArrayDComplex();
-    virtual Array<DComplex> getArrayDComplex (const vector<TableExprId>&);
+    virtual MArray<DComplex> getArrayDComplex (const vector<TableExprId>&);
   protected:
     // If not empty, check if the shape matches that of <src>itsValue</src>.
     // If <src>itsValue</src> is still empty, it is sized.
-    Bool checkShape (const ArrayBase& arr, const String& func);
-    Array<DComplex> itsValue;
+    Bool checkShape (const MArrayBase& arr, const String& func);
+    MArray<DComplex> itsValue;
   };
 
   // <summary>
@@ -717,12 +749,12 @@ namespace casacore { //# NAMESPACE CASACORE - BEGIN
       : TableExprGroupFuncBase (node)
     {}
     virtual ~TableExprGroupFuncArrayDate();
-    virtual Array<MVTime> getArrayDate (const vector<TableExprId>&);
+    virtual MArray<MVTime> getArrayDate (const vector<TableExprId>&);
   protected:
     // If not empty, check if the shape matches that of <src>itsValue</src>.
     // If <src>itsValue</src> is still empty, it is sized.
-    Bool checkShape (const ArrayBase& arr, const String& func);
-    Array<MVTime> itsValue;
+    Bool checkShape (const MArrayBase& arr, const String& func);
+    MArray<MVTime> itsValue;
   };
 
   // <summary>
@@ -745,12 +777,12 @@ namespace casacore { //# NAMESPACE CASACORE - BEGIN
       : TableExprGroupFuncBase (node)
     {}
     virtual ~TableExprGroupFuncArrayString();
-    virtual Array<String> getArrayString (const vector<TableExprId>&);
+    virtual MArray<String> getArrayString (const vector<TableExprId>&);
   protected:
     // If not empty, check if the shape matches that of <src>itsValue</src>.
     // If <src>itsValue</src> is still empty, it is sized.
-    Bool checkShape (const ArrayBase& arr, const String& func);
-    Array<String> itsValue;
+    Bool checkShape (const MArrayBase& arr, const String& func);
+    MArray<String> itsValue;
   };
 
 
