@@ -4,7 +4,7 @@
 //#
 //# This library is free software; you can redistribute it and/or modify it
 //# under the terms of the GNU Library General Public License as published by
-//# the Free Software Foundation; either version 2 of the License, or (at your
+//# the Free Software Foundatcd ion; either version 2 of the License, or (at your
 //# option) any later version.
 //#
 //# This library is distributed in the hope that it will be useful, but WITHOUT
@@ -37,6 +37,56 @@
 namespace casacore { //# NAMESPACE CASACORE - BEGIN
 
 
+TaQLConstNodeRep::TaQLConstNodeRep (Bool value)
+  : TaQLNodeRep (TaQLNode_Const),
+    itsType        (CTBool),
+    itsIsTableName (False),
+    itsBValue      (value)
+{}
+TaQLConstNodeRep::TaQLConstNodeRep (Int64 value, Bool isTableName)
+  : TaQLNodeRep (TaQLNode_Const),
+    itsType        (CTInt),
+    itsIsTableName (isTableName),
+    itsIValue      (value),
+    itsRValue      (value),
+    itsCValue      (value, 0.) 
+{}
+TaQLConstNodeRep::TaQLConstNodeRep (Double value)
+  : TaQLNodeRep (TaQLNode_Const),
+    itsType        (CTReal),
+    itsIsTableName (False),
+    itsRValue      (value),
+    itsCValue      (value, 0.)
+{}
+TaQLConstNodeRep::TaQLConstNodeRep (Double value, const String& unit)
+  : TaQLNodeRep (TaQLNode_Const),
+    itsType        (CTReal),
+    itsIsTableName (False),
+    itsRValue      (value),
+    itsCValue      (value, 0.),
+    itsUnit        (unit)
+{}
+TaQLConstNodeRep::TaQLConstNodeRep (DComplex value)
+  : TaQLNodeRep (TaQLNode_Const),
+    itsType        (CTComplex),
+    itsIsTableName (False),
+    itsCValue      (value)
+{}
+TaQLConstNodeRep::TaQLConstNodeRep (const String& value,
+                                    Bool isTableName)
+  : TaQLNodeRep (TaQLNode_Const),
+    itsType        (CTString), 
+    itsIsTableName (isTableName),
+    itsSValue      (value)
+{}
+TaQLConstNodeRep::TaQLConstNodeRep (const MVTime& value)
+  : TaQLNodeRep (TaQLNode_Const),
+    itsType        (CTTime),
+    itsIsTableName (False),
+    itsRValue      (value),
+    itsCValue      (value, 0.),
+    itsTValue      (value)
+{}
 TaQLConstNodeRep::~TaQLConstNodeRep()
 {}
 const String& TaQLConstNodeRep::getString() const
@@ -206,6 +256,16 @@ TaQLRegexNodeRep::TaQLRegexNodeRep (const String& regex)
     itsValue.downcase();
   }
 }
+TaQLRegexNodeRep::TaQLRegexNodeRep (const String& value,
+                                    Bool caseInsensitive, Bool negate,
+                                    Bool ignoreBlanks, Int maxDistance)
+  : TaQLNodeRep (TaQLNode_Regex),
+    itsValue           (value),
+    itsCaseInsensitive (caseInsensitive),
+    itsNegate          (negate),
+    itsIgnoreBlanks    (ignoreBlanks),
+    itsMaxDistance     (maxDistance)
+{}
 TaQLRegexNodeRep::~TaQLRegexNodeRep()
 {}
 TaQLNodeResult TaQLRegexNodeRep::visit (TaQLNodeVisitor& visitor) const
@@ -246,6 +306,11 @@ TaQLRegexNodeRep* TaQLRegexNodeRep::restore (AipsIO& aio)
 }
 
 
+TaQLUnaryNodeRep::TaQLUnaryNodeRep (Type type, const TaQLNode& child)
+  : TaQLNodeRep (TaQLNode_Unary),
+    itsType  (type),
+    itsChild (child)
+{}
 TaQLUnaryNodeRep::~TaQLUnaryNodeRep()
 {}
 TaQLNodeResult TaQLUnaryNodeRep::visit (TaQLNodeVisitor& visitor) const
@@ -294,6 +359,13 @@ TaQLUnaryNodeRep* TaQLUnaryNodeRep::restore (AipsIO& aio)
   return new TaQLUnaryNodeRep (type, node);
 }
 
+TaQLBinaryNodeRep::TaQLBinaryNodeRep (Type type, const TaQLNode& left,
+                                      const TaQLNode& right)
+  : TaQLNodeRep (TaQLNode_Binary),
+    itsType  (type),
+    itsLeft  (left),
+    itsRight (right)
+{}
 TaQLBinaryNodeRep::~TaQLBinaryNodeRep()
 {}
 TaQLBinaryNodeRep* TaQLBinaryNodeRep::handleRegex (const TaQLNode& left,
@@ -408,6 +480,22 @@ TaQLBinaryNodeRep* TaQLBinaryNodeRep::restore (AipsIO& aio)
   return new TaQLBinaryNodeRep (type, left, right);
 }
 
+TaQLMultiNodeRep::TaQLMultiNodeRep (Bool isSetOrArray)
+  : TaQLNodeRep (TaQLNode_Multi),
+    itsIsSetOrArray (isSetOrArray),
+    itsSep          (","),
+    itsIncr         (1)
+{}
+TaQLMultiNodeRep::TaQLMultiNodeRep(const String& prefix,
+                                   const String& postfix,
+                                   Bool isSetOrArray)
+  : TaQLNodeRep (TaQLNode_Multi),
+    itsIsSetOrArray (isSetOrArray),
+    itsPrefix       (prefix),
+    itsPostfix      (postfix),
+    itsSep          (","),
+    itsIncr         (1)
+{}
 TaQLMultiNodeRep::~TaQLMultiNodeRep()
 {}
 TaQLNodeResult TaQLMultiNodeRep::visit (TaQLNodeVisitor& visitor) const
@@ -419,7 +507,7 @@ void TaQLMultiNodeRep::show (std::ostream& os) const
   os << itsPrefix;
   for (uInt i=0; i<itsNodes.size(); ++i) {
     if (i != 0) {
-      os << ',';
+      os << (i%itsIncr == 0  ?  itsSep : itsSep2);
     }
     itsNodes[i].show (os);
   }
@@ -427,7 +515,8 @@ void TaQLMultiNodeRep::show (std::ostream& os) const
 }
 void TaQLMultiNodeRep::save (AipsIO& aio) const
 {
-  aio << itsIsSetOrArray << itsPrefix << itsPostfix;
+  aio << itsIsSetOrArray << itsPrefix << itsPostfix
+      << itsSep << itsSep2 << itsIncr;
   aio << uInt(itsNodes.size());
   for (uInt i=0; i<itsNodes.size(); ++i) {
     itsNodes[i].saveNode (aio);
@@ -435,18 +524,32 @@ void TaQLMultiNodeRep::save (AipsIO& aio) const
 }
 TaQLMultiNodeRep* TaQLMultiNodeRep::restore (AipsIO& aio)
 {
-  uInt size;
+  uInt size, incr;
   Bool isSetOrArray;
-  String prefix, postfix;
-  aio >> isSetOrArray >> prefix >> postfix;
+  String prefix, postfix, sep, sep2;
+  aio >> isSetOrArray >> prefix >> postfix
+      >> sep >> sep2 >> incr;
   aio >> size;
   TaQLMultiNodeRep* node = new TaQLMultiNodeRep(prefix, postfix, isSetOrArray);
+  node->setSeparator (sep);
+  node->setSeparator (incr, sep2);
   for (uInt i=0; i<size; ++i) {
     node->add (TaQLNode::restoreNode (aio));
   }
   return node;
 }
 
+TaQLFuncNodeRep::TaQLFuncNodeRep (const String& name)
+  : TaQLNodeRep (TaQLNode_Func),
+    itsName (name),
+    itsArgs (False)
+{}
+TaQLFuncNodeRep::TaQLFuncNodeRep (const String& name,
+                                  const TaQLMultiNode& args)
+  : TaQLNodeRep (TaQLNode_Func),
+    itsName (name),
+    itsArgs (args)
+{}
 TaQLFuncNodeRep::~TaQLFuncNodeRep()
 {}
 TaQLNodeResult TaQLFuncNodeRep::visit (TaQLNodeVisitor& visitor) const
@@ -471,6 +574,28 @@ TaQLFuncNodeRep* TaQLFuncNodeRep::restore (AipsIO& aio)
   return new TaQLFuncNodeRep (name, TaQLNode::restoreMultiNode (aio));
 }
 
+TaQLRangeNodeRep::TaQLRangeNodeRep (Bool leftClosed, TaQLNode start,
+                                    const TaQLNode& end, Bool rightClosed)
+  : TaQLNodeRep (TaQLNode_Range),
+    itsLeftClosed (leftClosed),
+    itsStart      (start),
+    itsEnd        (end),
+    itsRightClosed(rightClosed)
+{}
+TaQLRangeNodeRep::TaQLRangeNodeRep (Bool leftClosed, const TaQLNode& start)
+  : TaQLNodeRep (TaQLNode_Range),
+    itsLeftClosed (leftClosed),
+    itsStart      (start),
+    itsEnd        (),
+    itsRightClosed(False)
+{}
+TaQLRangeNodeRep::TaQLRangeNodeRep (const TaQLNode& end, Bool rightClosed)
+  : TaQLNodeRep (TaQLNode_Range),
+    itsLeftClosed (False),
+    itsStart      (),
+    itsEnd        (end),
+    itsRightClosed(rightClosed)
+{}
 TaQLRangeNodeRep::~TaQLRangeNodeRep()
 {}
 TaQLNodeResult TaQLRangeNodeRep::visit (TaQLNodeVisitor& visitor) const
@@ -508,6 +633,14 @@ TaQLRangeNodeRep* TaQLRangeNodeRep::restore (AipsIO& aio)
   return new TaQLRangeNodeRep (leftClosed, start, end, rightClosed);
 }
 
+TaQLIndexNodeRep::TaQLIndexNodeRep (const TaQLNode& start,
+                                    const TaQLNode& end,
+                                    const TaQLNode& incr)
+  : TaQLNodeRep (TaQLNode_Index),
+    itsStart (start),
+    itsEnd   (end),
+    itsIncr  (incr)
+{}
 TaQLIndexNodeRep::~TaQLIndexNodeRep()
 {}
 TaQLNodeResult TaQLIndexNodeRep::visit (TaQLNodeVisitor& visitor) const
@@ -542,6 +675,12 @@ TaQLIndexNodeRep* TaQLIndexNodeRep::restore (AipsIO& aio)
   return new TaQLIndexNodeRep (start, end, incr);
 }
 
+TaQLJoinNodeRep::TaQLJoinNodeRep (const TaQLMultiNode& tables,
+                                  const TaQLNode& condition)
+  : TaQLNodeRep (TaQLNode_Join),
+    itsTables    (tables),
+    itsCondition (condition)
+{}
 TaQLJoinNodeRep::~TaQLJoinNodeRep()
 {}
 TaQLNodeResult TaQLJoinNodeRep::visit (TaQLNodeVisitor& visitor) const
@@ -570,6 +709,12 @@ TaQLJoinNodeRep* TaQLJoinNodeRep::restore (AipsIO& aio)
   return new TaQLJoinNodeRep (tables, condition);
 }
 
+TaQLKeyColNodeRep::TaQLKeyColNodeRep (const String& name,
+                                      const String& nameMask)
+: TaQLNodeRep (TaQLNode_KeyCol),
+  itsName     (name),
+  itsNameMask (nameMask)
+{}
 TaQLKeyColNodeRep::~TaQLKeyColNodeRep()
 {}
 TaQLNodeResult TaQLKeyColNodeRep::visit (TaQLNodeVisitor& visitor) const
@@ -578,19 +723,29 @@ TaQLNodeResult TaQLKeyColNodeRep::visit (TaQLNodeVisitor& visitor) const
 }
 void TaQLKeyColNodeRep::show (std::ostream& os) const
 {
-  os << itsName;
+  if (itsNameMask.empty()) {
+    os << itsName;
+  } else {
+    os << '(' << itsName << ',' << itsNameMask << ')';
+  }
 }
 void TaQLKeyColNodeRep::save (AipsIO& aio) const
 {
-  aio << itsName;
+  aio << itsName << itsNameMask;
 }
 TaQLKeyColNodeRep* TaQLKeyColNodeRep::restore (AipsIO& aio)
 {
-  String name;
-  aio >> name;
-  return new TaQLKeyColNodeRep (name);
+  String name, nameMask;
+  aio >> name >> nameMask;
+  return new TaQLKeyColNodeRep (name, nameMask);
 }
 
+TaQLTableNodeRep::TaQLTableNodeRep (const TaQLNode& table,
+                                    const String& alias)
+  : TaQLNodeRep (TaQLNode_Table),
+    itsTable (table),
+    itsAlias (alias)
+{}
 TaQLTableNodeRep::~TaQLTableNodeRep()
 {}
 TaQLNodeResult TaQLTableNodeRep::visit (TaQLNodeVisitor& visitor) const
@@ -616,6 +771,14 @@ TaQLTableNodeRep* TaQLTableNodeRep::restore (AipsIO& aio)
   return new TaQLTableNodeRep (TaQLNode::restoreNode(aio), alias);
 }
 
+TaQLColNodeRep::TaQLColNodeRep (const TaQLNode& expr, const String& name,
+                                const String& nameMask, const String& dtype)
+  : TaQLNodeRep (TaQLNode_Col),
+    itsExpr     (expr),
+    itsName     (name),
+    itsNameMask (nameMask),
+    itsDtype    (checkDataType(dtype))
+{}
 TaQLColNodeRep::~TaQLColNodeRep()
 {}
 TaQLNodeResult TaQLColNodeRep::visit (TaQLNodeVisitor& visitor) const
@@ -626,7 +789,12 @@ void TaQLColNodeRep::show (std::ostream& os) const
 {
   itsExpr.show (os);
   if (! itsName.empty()) {
-    os << " AS " << itsName;
+    os << " AS " ;
+    if (itsNameMask.empty()) {
+      os << itsName;
+    } else {
+      os << '(' << itsName << ',' << itsNameMask << ')';
+    }
     if (! itsDtype.empty()) {
       os << ' ' << itsDtype;
     }
@@ -634,19 +802,24 @@ void TaQLColNodeRep::show (std::ostream& os) const
 }
 void TaQLColNodeRep::save (AipsIO& aio) const
 {
-  aio << itsName;
-  aio << itsDtype;
+  aio << itsName << itsNameMask << itsDtype;
   itsExpr.saveNode (aio);
 }
 TaQLColNodeRep* TaQLColNodeRep::restore (AipsIO& aio)
 {
-  String name, dtype;
-  aio >> name >> dtype;
+  String name, nameMask, dtype;
+  aio >> name >> nameMask >> dtype;
   TaQLColNodeRep* node = new TaQLColNodeRep (TaQLNode::restoreNode(aio),
-					     name, dtype);
+					     name, nameMask, dtype);
   return node;
 }
 
+TaQLColumnsNodeRep::TaQLColumnsNodeRep (Bool distinct,
+                                        const TaQLMultiNode& nodes)
+  : TaQLNodeRep (TaQLNode_Columns),
+    itsDistinct (distinct),
+    itsNodes    (nodes)
+{}
 TaQLColumnsNodeRep::~TaQLColumnsNodeRep()
 {}
 TaQLNodeResult TaQLColumnsNodeRep::visit (TaQLNodeVisitor& visitor) const
@@ -675,6 +848,11 @@ TaQLColumnsNodeRep* TaQLColumnsNodeRep::restore (AipsIO& aio)
   return new TaQLColumnsNodeRep (distinct, TaQLNode::restoreMultiNode(aio));
 }
 
+TaQLGroupNodeRep::TaQLGroupNodeRep (Type type, const TaQLMultiNode& nodes)
+  : TaQLNodeRep (TaQLNode_Groupby),
+    itsType  (type),
+    itsNodes (nodes)
+{}
 TaQLGroupNodeRep::~TaQLGroupNodeRep()
 {}
 TaQLNodeResult TaQLGroupNodeRep::visit (TaQLNodeVisitor& visitor) const
@@ -703,6 +881,11 @@ TaQLGroupNodeRep* TaQLGroupNodeRep::restore (AipsIO& aio)
   return new TaQLGroupNodeRep (type, TaQLNode::restoreMultiNode(aio));
 }
 
+TaQLSortKeyNodeRep::TaQLSortKeyNodeRep (Type type, const TaQLNode& child)
+  : TaQLNodeRep (TaQLNode_SortKey),
+    itsType  (type),
+    itsChild (child)
+{}
 TaQLSortKeyNodeRep::~TaQLSortKeyNodeRep()
 {}
 TaQLNodeResult TaQLSortKeyNodeRep::visit (TaQLNodeVisitor& visitor) const
@@ -736,6 +919,13 @@ TaQLSortKeyNodeRep* TaQLSortKeyNodeRep::restore (AipsIO& aio)
   return new TaQLSortKeyNodeRep (type, TaQLNode::restoreNode(aio));
 }
 
+TaQLSortNodeRep::TaQLSortNodeRep (Bool unique, Type type,
+                                  const TaQLMultiNode& keys)
+  : TaQLNodeRep (TaQLNode_Sort),
+    itsUnique (unique),
+    itsType   (type),
+    itsKeys   (keys)
+{}
 TaQLSortNodeRep::~TaQLSortNodeRep()
 {}
 TaQLNodeResult TaQLSortNodeRep::visit (TaQLNodeVisitor& visitor) const
@@ -768,6 +958,12 @@ TaQLSortNodeRep* TaQLSortNodeRep::restore (AipsIO& aio)
   return new TaQLSortNodeRep (unique, type, TaQLNode::restoreMultiNode(aio));
 }
 
+TaQLLimitOffNodeRep::TaQLLimitOffNodeRep (const TaQLNode& limit,
+                                          const TaQLNode& offset)
+  : TaQLNodeRep (TaQLNode_LimitOff),
+    itsLimit (limit),
+    itsOffset (offset)
+{}
 TaQLLimitOffNodeRep::~TaQLLimitOffNodeRep()
 {}
 TaQLNodeResult TaQLLimitOffNodeRep::visit (TaQLNodeVisitor& visitor) const
@@ -797,37 +993,16 @@ TaQLLimitOffNodeRep* TaQLLimitOffNodeRep::restore (AipsIO& aio)
   return new TaQLLimitOffNodeRep (limit, offset);
 }
 
-TaQLGivingNodeRep::TaQLGivingNodeRep (const String& name, const String& type)
+TaQLGivingNodeRep::TaQLGivingNodeRep (const String& name,
+                                      const TaQLMultiNode& type)
   : TaQLNodeRep (TaQLNode_Giving),
     itsName     (name),
-    itsType     (0)
-{
-  if (!type.empty()) {
-    String typel(type);
-    typel.downcase();
-    if (typel == "memory") {
-      itsType = 1;
-    } else if (typel == "scratch") {
-      itsType = 2;
-    } else if (typel == "plain") {
-      itsType = 3;
-    } else if (typel == "plain_big") {
-      itsType = 4;
-    } else if (typel == "plain_little") {
-      itsType = 5;
-    } else if (typel == "plain_local") {
-      itsType = 6;
-    } else {
-      throw TableParseError ("AS " + type + " in GIVING table " + name +
-			     " is invalid; "
-			     "use MEMORY or PLAIN[_BIG,LITTLE,LOCAL]");
-    }
-  }
-  if (itsName.empty()  &&  itsType > 2) {
-    throw TableParseError ("table name in GIVING can only be omitted if "
-                           "AS MEMORY or AS SCRATCH is given");
-  }
-}
+    itsType     (type)
+{}
+TaQLGivingNodeRep::TaQLGivingNodeRep (const TaQLMultiNode& exprlist)
+  : TaQLNodeRep (TaQLNode_Giving),
+    itsExprList (exprlist)
+{}
 TaQLGivingNodeRep::~TaQLGivingNodeRep()
 {}
 TaQLNodeResult TaQLGivingNodeRep::visit (TaQLNodeVisitor& visitor) const
@@ -836,44 +1011,22 @@ TaQLNodeResult TaQLGivingNodeRep::visit (TaQLNodeVisitor& visitor) const
 }
 void TaQLGivingNodeRep::show (std::ostream& os) const
 {
-  os << " GIVING ";
-  if (itsType < 0) {
+  if (itsExprList.isValid()) {
     itsExprList.show (os);
   } else {
     os << itsName;
-    if (itsType > 0) {
+    if (itsType.isValid()) {
       os << " AS ";
-      switch (itsType) {
-      case 1:
-	os << "memory";
-	break;
-      case 2:
-	os << "scratch";
-	break;
-      case 3:
-	os << "plain";
-	break;
-      case 4:
-	os << "plain_big";
-	break;
-      case 5:
-	os << "plain_little";
-	break;
-      case 6:
-	os << "plain_local";
-	break;
-      default:
-	os << "UNKNOWN";
-	break;
-      }
+      itsType.show (os);
     }
   }
 }
 void TaQLGivingNodeRep::save (AipsIO& aio) const
 {
   itsExprList.saveNode (aio);
-  if (itsType >= 0) {
-    aio << itsName << itsType;
+  if (! itsExprList.isValid()) {
+    aio << itsName;
+    itsType.saveNode (aio);
   }
 }
 TaQLGivingNodeRep* TaQLGivingNodeRep::restore (AipsIO& aio)
@@ -884,11 +1037,40 @@ TaQLGivingNodeRep* TaQLGivingNodeRep::restore (AipsIO& aio)
   }
   String name;
   aio >> name;
-  Int type;
-  aio >> type;
+  TaQLMultiNode type = TaQLNode::restoreMultiNode (aio);
   return new TaQLGivingNodeRep (name, type);
 }
 
+TaQLUpdExprNodeRep::TaQLUpdExprNodeRep (const String& name,
+                                        const String& nameMask,
+                                        const TaQLNode& expr)
+  : TaQLNodeRep (TaQLNode_UpdExpr),
+    itsName     (name),
+    itsNameMask (nameMask),
+    itsExpr     (expr)
+{}
+TaQLUpdExprNodeRep::TaQLUpdExprNodeRep (const String& name,
+                                        const String& nameMask,
+                                        const TaQLMultiNode& indices,
+                                        const TaQLNode& expr)
+  : TaQLNodeRep (TaQLNode_UpdExpr),
+    itsName     (name),
+    itsNameMask (nameMask),
+    itsIndices1 (indices),
+    itsExpr     (expr)
+{}
+TaQLUpdExprNodeRep::TaQLUpdExprNodeRep (const String& name,
+                                        const String& nameMask,
+                                        const TaQLMultiNode& indices1,
+                                        const TaQLMultiNode& indices2,
+                                        const TaQLNode& expr)
+  : TaQLNodeRep (TaQLNode_UpdExpr),
+    itsName     (name),
+    itsNameMask (nameMask),
+    itsIndices1 (indices1),
+    itsIndices2 (indices2),
+    itsExpr     (expr)
+{}
 TaQLUpdExprNodeRep::~TaQLUpdExprNodeRep()
 {}
 TaQLNodeResult TaQLUpdExprNodeRep::visit (TaQLNodeVisitor& visitor) const
@@ -897,24 +1079,31 @@ TaQLNodeResult TaQLUpdExprNodeRep::visit (TaQLNodeVisitor& visitor) const
 }
 void TaQLUpdExprNodeRep::show (std::ostream& os) const
 {
-  os << itsName;
-  itsIndices.show (os);
+  if (itsNameMask.empty()) {
+    os << itsName;
+  } else {
+    os << '(' << itsName << ',' << itsNameMask << ')';
+  }
+  itsIndices1.show (os);
+  itsIndices2.show (os);
   os << '=';
   itsExpr.show (os);
 }
 void TaQLUpdExprNodeRep::save (AipsIO& aio) const
 {
-  aio << itsName;
-  itsIndices.saveNode (aio);
+  aio << itsName << itsNameMask;
+  itsIndices1.saveNode (aio);
+  itsIndices2.saveNode (aio);
   itsExpr.saveNode (aio);
 }
 TaQLUpdExprNodeRep* TaQLUpdExprNodeRep::restore (AipsIO& aio)
 {
-  String name;
-  aio >> name;
-  TaQLMultiNode indices = TaQLNode::restoreMultiNode (aio);
+  String name, nameMask;
+  aio >> name >> nameMask;
+  TaQLMultiNode indices1 = TaQLNode::restoreMultiNode (aio);
+  TaQLMultiNode indices2 = TaQLNode::restoreMultiNode (aio);
   TaQLNode expr = TaQLNode::restoreNode (aio);
-  return new TaQLUpdExprNodeRep (name, indices, expr);
+  return new TaQLUpdExprNodeRep (name, nameMask, indices1, indices2, expr);
 }
 
 TaQLQueryNodeRep::TaQLQueryNodeRep (int nodeType)
@@ -944,6 +1133,20 @@ void TaQLQueryNodeRep::restoreSuper (AipsIO& aio)
   aio >> itsBrackets >> itsNoExecute >> itsFromExecute;
 }
 
+TaQLSelectNodeRep::TaQLSelectNodeRep (const TaQLNode& columns,
+                                      const TaQLNode& where,
+                                      const TaQLNode& groupby,
+                                      const TaQLNode& having,
+                                      const TaQLNode& sort,
+                                      const TaQLNode& limitoff,
+                                      const TaQLNode& giving,
+                                      const TaQLMultiNode& dminfo)
+  : TaQLQueryNodeRep (TaQLNode_Select),
+    itsColumns(columns),
+    itsWhere(where), itsGroupby(groupby), itsHaving(having),
+    itsSort(sort), itsLimitOff(limitoff), itsGiving(giving),
+    itsDMInfo(dminfo)
+{} 
 TaQLSelectNodeRep::TaQLSelectNodeRep (const TaQLNode& columns,
                                       const TaQLMultiNode& tables,
                                       const TaQLNode& join,
@@ -952,11 +1155,13 @@ TaQLSelectNodeRep::TaQLSelectNodeRep (const TaQLNode& columns,
                                       const TaQLNode& having,
                                       const TaQLNode& sort,
                                       const TaQLNode& limitoff,
-                                      const TaQLNode& giving)
+                                      const TaQLNode& giving,
+                                      const TaQLMultiNode& dminfo)
   : TaQLQueryNodeRep (TaQLNode_Select),
     itsColumns(columns), itsTables(tables), itsJoin(join),
     itsWhere(where), itsGroupby(groupby), itsHaving(having),
-    itsSort(sort), itsLimitOff(limitoff), itsGiving(giving)
+    itsSort(sort), itsLimitOff(limitoff), itsGiving(giving),
+    itsDMInfo(dminfo)
 {}
 TaQLSelectNodeRep::~TaQLSelectNodeRep()
 {}
@@ -968,15 +1173,16 @@ void TaQLSelectNodeRep::showDerived (std::ostream& os) const
 {
   os << "SELECT";
   itsColumns.show (os);
-  os << " FROM ";
-  itsTables.show (os);
+  if (itsTables.isValid()) {
+    os << " FROM ";
+    itsTables.show (os);
+  }
   itsJoin.show (os);
   if (itsWhere.isValid()) {
     os << " WHERE ";
     itsWhere.show (os);
   }
   if (itsGroupby.isValid()) {
-    os << " GROUPBY ";
     itsGroupby.show (os);
   }
   if (itsHaving.isValid()) {
@@ -985,7 +1191,14 @@ void TaQLSelectNodeRep::showDerived (std::ostream& os) const
   }
   itsSort.show (os);
   itsLimitOff.show (os);
-  itsGiving.show (os);
+  if (itsGiving.isValid()) {
+    os << " GIVING ";
+    itsGiving.show (os);
+  }
+  if (itsDMInfo.isValid()) {
+    os << " DMINFO ";
+    itsDMInfo.show (os);
+  }
 }
 void TaQLSelectNodeRep::save (AipsIO& aio) const
 {
@@ -998,6 +1211,7 @@ void TaQLSelectNodeRep::save (AipsIO& aio) const
   itsSort.saveNode (aio);
   itsLimitOff.saveNode (aio);
   itsGiving.saveNode (aio);
+  itsDMInfo.saveNode (aio);
   saveSuper (aio);
 }
 TaQLSelectNodeRep* TaQLSelectNodeRep::restore (AipsIO& aio)
@@ -1006,14 +1220,16 @@ TaQLSelectNodeRep* TaQLSelectNodeRep::restore (AipsIO& aio)
   TaQLMultiNode tables = TaQLNode::restoreMultiNode (aio);
   TaQLNode join = TaQLNode::restoreMultiNode (aio);
   TaQLNode where = TaQLNode::restoreNode (aio);
-  TaQLNode groupby = TaQLNode::restoreMultiNode (aio);
-  TaQLNode having = TaQLNode::restoreMultiNode (aio);
+  TaQLNode groupby = TaQLNode::restoreNode (aio);
+  TaQLNode having = TaQLNode::restoreNode (aio);
   TaQLNode sort = TaQLNode::restoreNode (aio);
   TaQLNode limitoff = TaQLNode::restoreNode (aio);
   TaQLNode giving = TaQLNode::restoreNode (aio);
+  TaQLMultiNode dminfo = TaQLNode::restoreMultiNode (aio);
   TaQLSelectNodeRep* node = new TaQLSelectNodeRep (columns, tables, join,
 						   where, groupby, having,
-						   sort, limitoff, giving);
+						   sort, limitoff, giving,
+                                                   dminfo);
   node->restoreSuper (aio);
   return node;
 }
@@ -1058,6 +1274,20 @@ TaQLCountNodeRep* TaQLCountNodeRep::restore (AipsIO& aio)
   return node;
 }
 
+TaQLUpdateNodeRep::TaQLUpdateNodeRep (const TaQLMultiNode& tables,
+                                      const TaQLMultiNode& update,
+                                      const TaQLMultiNode& from,
+                                      const TaQLNode& where,
+                                      const TaQLNode& sort,
+                                      const TaQLNode& limitoff)
+  : TaQLNodeRep (TaQLNode_Update),
+    itsTables   (tables),
+    itsUpdate   (update),
+    itsFrom     (from),
+    itsWhere    (where),
+    itsSort     (sort),
+    itsLimitOff (limitoff)
+{}
 TaQLUpdateNodeRep::~TaQLUpdateNodeRep()
 {}
 TaQLNodeResult TaQLUpdateNodeRep::visit (TaQLNodeVisitor& visitor) const
@@ -1101,11 +1331,21 @@ TaQLUpdateNodeRep* TaQLUpdateNodeRep::restore (AipsIO& aio)
   return new TaQLUpdateNodeRep (tables, update, from, where, sort, limitoff);
 }
 
+TaQLInsertNodeRep::TaQLInsertNodeRep (const TaQLMultiNode& tables,
+                                      const TaQLMultiNode& columns,
+                                      const TaQLNode& values,
+                                      const TaQLNode& limit)
+  : TaQLNodeRep (TaQLNode_Insert),
+    itsTables  (tables),
+    itsColumns (columns),
+    itsValues  (values),
+    itsLimit   (limit)
+{}
 TaQLInsertNodeRep::TaQLInsertNodeRep (const TaQLMultiNode& tables,
                                       const TaQLMultiNode& insert)
   : TaQLNodeRep (TaQLNode_Insert),
-    itsTables   (tables),
-    itsColumns  (False)
+    itsTables  (tables),
+    itsColumns (False)
 {
   // Convert the list of column=value expressions like
   //        SET col1=val1, col2=val2
@@ -1119,15 +1359,17 @@ TaQLInsertNodeRep::TaQLInsertNodeRep (const TaQLMultiNode& tables,
     const TaQLUpdExprNodeRep* rep = dynamic_cast<const TaQLUpdExprNodeRep*>
       (nodes[i].getRep());
     AlwaysAssert (rep, AipsError);
-    if (rep->itsIndices.isValid()) {
-      throw TableInvExpr ("Column indices cannot be given in an "
+    if (rep->itsIndices1.isValid()  ||  rep->itsIndices2.isValid()) {
+      throw TableInvExpr ("Column indices or masks cannot be given in an "
                           "INSERT command");
     }
     // Add the column name and value expression.
     itsColumns.add (new TaQLKeyColNodeRep (rep->itsName));
     values.add (rep->itsExpr);
   }
-  itsValues = values;
+  TaQLMultiNode valuesList(False);
+  valuesList.add (values);
+  itsValues = valuesList;
 }
 TaQLInsertNodeRep::~TaQLInsertNodeRep()
 {}
@@ -1137,7 +1379,12 @@ TaQLNodeResult TaQLInsertNodeRep::visit (TaQLNodeVisitor& visitor) const
 }
 void TaQLInsertNodeRep::show (std::ostream& os) const
 {
-  os << "INSERT INTO ";
+  os << "INSERT";
+  if (itsLimit.isValid()) {
+    os << " LIMIT ";
+    itsLimit.show (os);
+  }
+  os << " INTO ";
   itsTables.show (os);
   if (itsColumns.isValid()) {
     os << " [";
@@ -1152,15 +1399,27 @@ void TaQLInsertNodeRep::save (AipsIO& aio) const
   itsTables.saveNode (aio);
   itsColumns.saveNode (aio);
   itsValues.saveNode (aio);
+  itsLimit.saveNode (aio);
 }
 TaQLInsertNodeRep* TaQLInsertNodeRep::restore (AipsIO& aio)
 {
   TaQLMultiNode tables = TaQLNode::restoreMultiNode (aio);
   TaQLMultiNode columns = TaQLNode::restoreMultiNode (aio);
   TaQLNode values = TaQLNode::restoreNode (aio);
-  return new TaQLInsertNodeRep (tables, columns, values);
+  TaQLNode limit  = TaQLNode::restoreNode (aio);
+  return new TaQLInsertNodeRep (tables, columns, values, limit);
 }
 
+TaQLDeleteNodeRep::TaQLDeleteNodeRep (const TaQLMultiNode& tables,
+                                      const TaQLNode& where,
+                                      const TaQLNode& sort,
+                                      const TaQLNode& limitoff)
+  : TaQLNodeRep (TaQLNode_Delete),
+    itsTables   (tables),
+    itsWhere    (where),
+    itsSort     (sort),
+    itsLimitOff (limitoff)
+{}
 TaQLDeleteNodeRep::~TaQLDeleteNodeRep()
 {}
 TaQLNodeResult TaQLDeleteNodeRep::visit (TaQLNodeVisitor& visitor) const
@@ -1194,6 +1453,18 @@ TaQLDeleteNodeRep* TaQLDeleteNodeRep::restore (AipsIO& aio)
   return new TaQLDeleteNodeRep (tables, where, sort, limitoff);
 }
 
+TaQLCalcNodeRep::TaQLCalcNodeRep (const TaQLMultiNode& tables,
+                                  const TaQLNode& expr,
+                                  const TaQLNode& where,
+                                  const TaQLNode& sort,
+                                  const TaQLNode& limitoff)
+  : TaQLNodeRep (TaQLNode_Calc),
+    itsTables   (tables),
+    itsExpr     (expr),
+    itsWhere    (where),
+    itsSort     (sort),
+    itsLimitOff (limitoff)
+{}
 TaQLCalcNodeRep::~TaQLCalcNodeRep()
 {}
 TaQLNodeResult TaQLCalcNodeRep::visit (TaQLNodeVisitor& visitor) const
@@ -1233,36 +1504,65 @@ TaQLCalcNodeRep* TaQLCalcNodeRep::restore (AipsIO& aio)
   return new TaQLCalcNodeRep (tables, expr, where, sort, limitoff);
 }
 
+TaQLCreTabNodeRep::TaQLCreTabNodeRep (const TaQLNode& giving,
+                                      const TaQLMultiNode& cols,
+                                      const TaQLNode& limit,
+                                      const TaQLMultiNode& dminfo)
+  : TaQLQueryNodeRep (TaQLNode_CreTab),
+    itsGiving  (giving),
+    itsColumns (cols),
+    itsLimit   (limit),
+    itsDMInfo  (dminfo)
+{}
 TaQLCreTabNodeRep::~TaQLCreTabNodeRep()
 {}
 TaQLNodeResult TaQLCreTabNodeRep::visit (TaQLNodeVisitor& visitor) const
 {
   return visitor.visitCreTabNode (*this);
 }
-void TaQLCreTabNodeRep::show (std::ostream& os) const
+void TaQLCreTabNodeRep::showDerived (std::ostream& os) const
 {
-  os << "CREATE TABLE " << itsName << ' ';
+  os << "CREATE TABLE ";
+  itsGiving.show (os);
+  os << ' ';
   itsColumns.show (os);
-  if (itsDataMans.isValid()) {
+  if (itsLimit.isValid()) {
+    os << " LIMIT ";
+    itsLimit.show (os);
+  }
+  if (itsDMInfo.isValid()) {
     os << " DMINFO ";
-    itsDataMans.show (os);
+    itsDMInfo.show (os);
   }
 }
 void TaQLCreTabNodeRep::save (AipsIO& aio) const
 {
-  aio << itsName;
+  itsGiving.saveNode (aio);
   itsColumns.saveNode (aio);
-  itsDataMans.saveNode (aio);
+  itsLimit.saveNode (aio);
+  itsDMInfo.saveNode (aio);
+  saveSuper (aio);
 }
 TaQLCreTabNodeRep* TaQLCreTabNodeRep::restore (AipsIO& aio)
 {
-  String name;
-  aio >> name;
+  TaQLNode giving = TaQLNode::restoreNode (aio);
   TaQLMultiNode columns = TaQLNode::restoreMultiNode (aio);
-  TaQLMultiNode datamans = TaQLNode::restoreMultiNode (aio);
-  return new TaQLCreTabNodeRep (name, columns, datamans);
+  TaQLNode limit = TaQLNode::restoreNode (aio);
+  TaQLMultiNode dminfo = TaQLNode::restoreMultiNode (aio);
+  TaQLCreTabNodeRep* node = new TaQLCreTabNodeRep (giving, columns,
+                                                   limit, dminfo);
+  node->restoreSuper (aio);
+  return node;
 }
 
+TaQLColSpecNodeRep::TaQLColSpecNodeRep (const String& name,
+                                        const String& dtype,
+                                        const TaQLMultiNode& spec)
+  : TaQLNodeRep (TaQLNode_ColSpec),
+    itsName  (name),
+    itsDtype (checkDataType(dtype)),
+    itsSpec  (spec)
+{}
 TaQLColSpecNodeRep::~TaQLColSpecNodeRep()
 {}
 TaQLNodeResult TaQLColSpecNodeRep::visit (TaQLNodeVisitor& visitor) const
@@ -1293,6 +1593,29 @@ TaQLColSpecNodeRep* TaQLColSpecNodeRep::restore (AipsIO& aio)
   return new TaQLColSpecNodeRep (name, dtype, spec);
 }
 
+TaQLRecFldNodeRep::TaQLRecFldNodeRep (const String& name,
+                                      const TaQLNode& values,
+                                      const String& dtype)
+  : TaQLNodeRep (TaQLNode_RecFld),
+    itsName  (name),
+    itsDtype (checkDataType(dtype)),
+    itsValues(values)
+{}
+TaQLRecFldNodeRep::TaQLRecFldNodeRep (const String& name,
+                                      const TaQLRecFldNodeRep& node)
+  : TaQLNodeRep (TaQLNode_RecFld),
+    itsName  (name),
+    itsDtype (node.itsDtype),
+    itsValues(node.itsValues)
+{}
+TaQLRecFldNodeRep::TaQLRecFldNodeRep (const String& name,
+                                      const String& fromName,
+                                      const String& dtype)
+  : TaQLNodeRep (TaQLNode_RecFld),
+    itsName     (name),
+    itsFromName (fromName),
+    itsDtype    (checkDataType(dtype))
+{}
 TaQLRecFldNodeRep::~TaQLRecFldNodeRep()
 {}
 TaQLNodeResult TaQLRecFldNodeRep::visit (TaQLNodeVisitor& visitor) const
@@ -1304,21 +1627,45 @@ void TaQLRecFldNodeRep::show (std::ostream& os) const
   if (! itsName.empty()) {
     os << itsName << '=';
   }
-  itsValues.show (os);
+  if (! itsFromName.empty()) {
+    os << itsFromName;
+  } else if (itsValues.isValid()) {
+    if (itsValues.nodeType() == TaQLNode_Multi  &&
+        ((const TaQLMultiNodeRep*)(itsValues.getRep()))->itsNodes.empty()) {
+      os << "[=]";
+    } else {
+      itsValues.show (os);
+    }
+  } else {
+    os << "[]";
+  }
+  if (! itsDtype.empty()) {
+    os << " AS " << itsDtype;
+  }
 }
 void TaQLRecFldNodeRep::save (AipsIO& aio) const
 {
-  aio << itsName;
+  aio << itsName << itsFromName << itsDtype;
   itsValues.saveNode (aio);
 }
 TaQLRecFldNodeRep* TaQLRecFldNodeRep::restore (AipsIO& aio)
 {
-  String name;
-  aio >> name;
+  String name, fromName, dtype;
+  aio >> name >> fromName >> dtype;
   TaQLNode values = TaQLNode::restoreNode (aio);
-  return new TaQLRecFldNodeRep (name, values);
+  if (fromName.empty()) {
+    return new TaQLRecFldNodeRep (name, values, dtype);
+  } else {
+    return new TaQLRecFldNodeRep (name, fromName, dtype);
+  }
 }
 
+TaQLUnitNodeRep::TaQLUnitNodeRep (const String& unit,
+                                  const TaQLNode& child)
+  : TaQLNodeRep (TaQLNode_Unit),
+    itsUnit  (unit),
+    itsChild (child)
+{}
 TaQLUnitNodeRep::~TaQLUnitNodeRep()
 {}
 TaQLNodeResult TaQLUnitNodeRep::visit (TaQLNodeVisitor& visitor) const
@@ -1344,5 +1691,232 @@ TaQLUnitNodeRep* TaQLUnitNodeRep::restore (AipsIO& aio)
   return new TaQLUnitNodeRep (unit, node);
 }
 
+TaQLAltTabNodeRep::TaQLAltTabNodeRep (const TaQLNode& table,
+                                      const TaQLMultiNode& from,
+                                      const TaQLMultiNode& commands)
+  : TaQLQueryNodeRep (TaQLNode_AltTab),
+    itsTable    (table),
+    itsFrom     (from),
+    itsCommands (commands)
+{}
+TaQLAltTabNodeRep::~TaQLAltTabNodeRep()
+{}
+TaQLNodeResult TaQLAltTabNodeRep::visit (TaQLNodeVisitor& visitor) const
+{
+  return visitor.visitAltTabNode (*this);
+}
+void TaQLAltTabNodeRep::showDerived (std::ostream& os) const
+{
+  os << "ALTER TABLE ";
+  itsTable.show (os);
+  if (itsFrom.isValid()) {
+    os << " FROM ";
+    itsFrom.show (os);
+  }
+  os << ' ';
+  itsCommands.show (os);
+}
+void TaQLAltTabNodeRep::save (AipsIO& aio) const
+{
+  itsTable.saveNode (aio);
+  itsFrom.saveNode (aio);
+  itsCommands.saveNode (aio);
+}
+TaQLAltTabNodeRep* TaQLAltTabNodeRep::restore (AipsIO& aio)
+{
+  TaQLNode table         = TaQLNode::restoreNode (aio);
+  TaQLMultiNode from     = TaQLNode::restoreMultiNode (aio);
+  TaQLMultiNode commands = TaQLNode::restoreMultiNode (aio);
+  return new TaQLAltTabNodeRep (table, from, commands);
+}
+
+TaQLAddColNodeRep::TaQLAddColNodeRep (const TaQLMultiNode& cols,
+                                      const TaQLMultiNode& dminfo)
+  : TaQLNodeRep (TaQLNode_AddCol),
+    itsColumns (cols),
+    itsDMInfo  (dminfo)
+{}
+TaQLAddColNodeRep::~TaQLAddColNodeRep()
+{}
+TaQLNodeResult TaQLAddColNodeRep::visit (TaQLNodeVisitor& visitor) const
+{
+  return visitor.visitAddColNode (*this);
+}
+void TaQLAddColNodeRep::show (std::ostream& os) const
+{
+  os << "ADD COLUMN ";
+  itsColumns.show (os);
+  if (itsDMInfo.isValid()) {
+    os << " DMINFO ";
+    itsDMInfo.show (os);
+  }
+}
+void TaQLAddColNodeRep::save (AipsIO& aio) const
+{
+  itsColumns.saveNode(aio);
+  itsDMInfo.saveNode(aio);
+}
+TaQLAddColNodeRep* TaQLAddColNodeRep::restore (AipsIO& aio)
+{
+  TaQLMultiNode cols   = TaQLNode::restoreMultiNode (aio);
+  TaQLMultiNode dminfo = TaQLNode::restoreMultiNode (aio);
+  return new TaQLAddColNodeRep (cols, dminfo);
+}
+
+TaQLRenDropNodeRep::TaQLRenDropNodeRep (Int type, const TaQLMultiNode& names)
+  : TaQLNodeRep (TaQLNode_RenDrop),
+    itsType  (type),
+    itsNames (names)
+{}
+TaQLRenDropNodeRep::~TaQLRenDropNodeRep()
+{}
+TaQLNodeResult TaQLRenDropNodeRep::visit (TaQLNodeVisitor& visitor) const
+{
+  return visitor.visitRenDropNode (*this);
+}
+void TaQLRenDropNodeRep::show (std::ostream& os) const
+{
+  if (itsType == 0) {
+    os << "RENAME COLUMN ";
+  } else if (itsType == 1) {
+    os << "DROP COLUMN ";
+  } else if (itsType == 2) {
+    os << "RENAME KEYWORD ";
+  } else {
+    os << "DROP KEYWORD ";
+  }
+  itsNames.show (os);
+}
+void TaQLRenDropNodeRep::save (AipsIO& aio) const
+{
+  aio << itsType;
+  itsNames.saveNode (aio);
+}
+TaQLRenDropNodeRep* TaQLRenDropNodeRep::restore (AipsIO& aio)
+{
+  Int type;
+  aio >> type;
+  TaQLMultiNode names = TaQLNode::restoreMultiNode (aio);
+  return new TaQLRenDropNodeRep (type, names);
+}
+
+TaQLSetKeyNodeRep::TaQLSetKeyNodeRep (const TaQLMultiNode& keyvals)
+  : TaQLNodeRep (TaQLNode_SetKey),
+    itsKeyVals (keyvals)
+{}
+TaQLSetKeyNodeRep::~TaQLSetKeyNodeRep()
+{}
+TaQLNodeResult TaQLSetKeyNodeRep::visit (TaQLNodeVisitor& visitor) const
+{
+  return visitor.visitSetKeyNode (*this);
+}
+void TaQLSetKeyNodeRep::show (std::ostream& os) const
+{
+  os << "SET KEYWORD ";
+  itsKeyVals.show (os);
+}
+void TaQLSetKeyNodeRep::save (AipsIO& aio) const
+{
+  itsKeyVals.saveNode (aio);
+}
+TaQLSetKeyNodeRep* TaQLSetKeyNodeRep::restore (AipsIO& aio)
+{
+  TaQLMultiNode keyvals = TaQLNode::restoreMultiNode (aio);
+  return new TaQLSetKeyNodeRep (keyvals);
+}
+
+TaQLAddRowNodeRep::TaQLAddRowNodeRep (const TaQLNode& nrow)
+  : TaQLNodeRep (TaQLNode_AddRow),
+    itsNRow(nrow)
+{}
+TaQLAddRowNodeRep::~TaQLAddRowNodeRep()
+{}
+TaQLNodeResult TaQLAddRowNodeRep::visit (TaQLNodeVisitor& visitor) const
+{
+  return visitor.visitAddRowNode (*this);
+}
+void TaQLAddRowNodeRep::show (std::ostream& os) const
+{
+  os << "ADD ROW ";
+  itsNRow.show (os);
+}
+void TaQLAddRowNodeRep::save (AipsIO& aio) const
+{
+  itsNRow.saveNode (aio);
+}
+TaQLAddRowNodeRep* TaQLAddRowNodeRep::restore (AipsIO& aio)
+{
+  TaQLNode nrow = TaQLNode::restoreNode (aio);
+  return new TaQLAddRowNodeRep (nrow);
+}
+
+TaQLConcTabNodeRep::TaQLConcTabNodeRep (const String& tableName,
+                                        const TaQLMultiNode& tables,
+                                        const TaQLMultiNode& subtableNames)
+  : TaQLQueryNodeRep (TaQLNode_ConcTab),
+    itsTableName (tableName),
+    itsTables    (tables),
+    itsSubTables (subtableNames)
+{}
+TaQLConcTabNodeRep::~TaQLConcTabNodeRep()
+{}
+TaQLNodeResult TaQLConcTabNodeRep::visit (TaQLNodeVisitor& visitor) const
+{
+  return visitor.visitConcTabNode (*this);
+}
+void TaQLConcTabNodeRep::showDerived (std::ostream& os) const
+{
+  os << "[";
+  itsTables.show (os);
+  if (itsSubTables.isValid()) {
+    os << " SUBTABLES ";
+    itsSubTables.show (os);
+  }
+  if (! itsTableName.empty()) {
+    os << " GIVING " << itsTableName;
+  }
+  os << ']';
+}
+void TaQLConcTabNodeRep::save (AipsIO& aio) const
+{
+  aio << itsTableName;
+  itsTables.saveNode (aio);
+  itsSubTables.saveNode (aio);
+}
+TaQLConcTabNodeRep* TaQLConcTabNodeRep::restore (AipsIO& aio)
+{
+  String tableName;
+  aio >> tableName;
+  TaQLMultiNode tables = TaQLNode::restoreMultiNode (aio);
+  TaQLMultiNode subtables = TaQLNode::restoreMultiNode (aio);
+  return new TaQLConcTabNodeRep (tableName, tables, subtables);
+}
+
+TaQLShowNodeRep::TaQLShowNodeRep (const TaQLMultiNode& names)
+  : TaQLNodeRep (TaQLNode_Show),
+    itsNames (names)
+{}
+TaQLShowNodeRep::~TaQLShowNodeRep()
+{}
+TaQLNodeResult TaQLShowNodeRep::visit (TaQLNodeVisitor& visitor) const
+{
+  return visitor.visitShowNode (*this);
+}
+void TaQLShowNodeRep::show (std::ostream& os) const
+{
+  if (itsNames.isValid()) {
+    itsNames.show (os);
+  }
+}
+void TaQLShowNodeRep::save (AipsIO& aio) const
+{
+  itsNames.saveNode (aio);
+}
+TaQLShowNodeRep* TaQLShowNodeRep::restore (AipsIO& aio)
+{
+  TaQLMultiNode names = TaQLNode::restoreMultiNode (aio);
+  return new TaQLShowNodeRep (names);
+}
+
 
 } //# NAMESPACE CASACORE - END
